@@ -100,7 +100,7 @@ my $execpath = defined $ENV{AUTOPKGTEST_TMP} ? "/usr/bin" : "perl ..";
 
 # msginit annoyances, see https://github.com/mquinson/po4a/issues/338
 my $PODIFF =
-    "-I'^\"Project-Id-Version:' -I'^\"POT-Creation-Date:' -I'^\"PO-Revision-Date:' "
+    "--ignore-trailing-space -I'^\"Project-Id-Version:' -I'^\"POT-Creation-Date:' -I'^\"PO-Revision-Date:' "
   . "-I'^# [^[:blank:]]* translations for ' -I'^# Language [^[:blank:]]* translations for ' -I'Copyright (C) 20.. Free Software Foundation, Inc.' -I'^# This file is distributed under the same license as the' -I'^# Automatically generated, 20...' ";
 
 sub show_files {
@@ -476,8 +476,9 @@ sub run_one_format {
     unless ($error or exists $test->{'skip'}{'updatepo'}) {
         # Update PO
         copy( "$cwd/$pofile", "$cwd/${tmpbase}.po_updated" ) || fail "Cannot copy $pofile before updating it";
+        my $updatepo_options = $options =~ s/--width= [-]? [0-9]+//rxms;
         my $cmd =
-            "${execpath}/po4a-updatepo --no-deprecation -f $format $options "
+            "${execpath}/po4a-updatepo --no-deprecation -f $format $updatepo_options "
           . "--master $basename.$ext --po $cwd/${tmpbase}.po_updated"
           . " > $cwd/tmp/$path/update.stderr 2>&1";
 
